/*
 * radio_alarm.h
 *
 *  Created on: Sep 18, 2010
 *      Author: Thomas Schmid
 */

#ifndef RADIO_ALARM_H_
#define RADIO_ALARM_H_

#include "../mss_timer/mss_timer.h"

#define RADIO_ALARM_MICROSEC 1

uint32_t tim1_clk;

typedef enum alarm_state {
	BUSY,
	FREE
} alarm_state_t;

alarm_state_t alarm_state;

typedef void (*radioalarm_handler_t)(void);
radioalarm_handler_t radio_alarm_fired;

__attribute__((__interrupt__)) void Timer1_IRQHandler()
{
	alarm_state = FREE;
	MSS_TIM1_disable_irq();
    MSS_TIM1_clear_irq();
    NVIC_ClearPendingIRQ( Timer1_IRQn );
	(*(radio_alarm_fired))(); // call the callback function
}

void RadioAlarm_init(radioalarm_handler_t handler)
{
	MSS_TIM1_init(MSS_TIMER_ONE_SHOT_MODE);
	MSS_TIM1_stop();
	MSS_TIM1_disable_irq();

	SystemCoreClockUpdate();
	tim1_clk = g_FrequencyPCLK0/1e6;
	radio_alarm_fired = handler;
	alarm_state = FREE;
}

void RadioAlarm_wait(uint32_t time){
	alarm_state = BUSY;
	MSS_TIM1_load_immediate(time*tim1_clk);
	MSS_TIM1_enable_irq();
	MSS_TIM1_start();
}
uint8_t RadioAlarm_isFree()
{
	return alarm_state == FREE;
}

uint8_t RadioAlarm_getNow()
{
	return MSS_TIM1_get_current_value();
}

#endif /* RADIO_ALARM_H_ */
